﻿/*	Application		A program which allocates memory in 100,000 blocks each time the user presses a key.
	File			DynamicMemoryAllocation.cpp 
	Purpose			Demonstrates dynamic memory allocation
	Author			Richard John Anthony
	Date			January 2015
	
	Function		1. Runs in a loop requesting memory, finds allocation limit by doubling request size until a request is refused
					2. Requests and holds the maximum allocation memory amount discovered.
					3. Displays the amount of memory held
*/

using System;
using System.Runtime.InteropServices;

namespace DynamicMemoryAllocation
{
    class Program
    {
        public const Int32 iAllocationIncrement = 100000;	// 100,000 bytes
        
        static void Main(string[] args)
        {
            string sInput;
	        IntPtr piMemoryAllocated;
	        Int32 iTotalMemoryAllocated = 0;
	        Console.WriteLine("*******************************");
	        Console.WriteLine("*  Dynamic Memory Allocation  *");
	        Console.WriteLine("*******************************");
	
	        while(true)
	        {
		        Console.WriteLine("Type a key to allocate a 100,000 byte block of memory or type \"Quit\" to quit: ");
		        sInput = Console.ReadLine();
 		        if("QUIT" == sInput.ToUpper())
		        {
			        Console.WriteLine("Quiting");
		            System.Environment.Exit(1);
		        }

		        Console.WriteLine("Requesting {0} bytes", iAllocationIncrement);
		        try
                {
                    piMemoryAllocated = Marshal.AllocHGlobal(iAllocationIncrement); // The equivallent of C malloc
			        iTotalMemoryAllocated += iAllocationIncrement;
			        Console.WriteLine("Result:ALLOCATED!  (Total now is: {0}", iTotalMemoryAllocated);
		        }
                catch(OutOfMemoryException ex1) // There is insufficient memory to satisfy the request.
		        {
			        Console.WriteLine("Result:REFUSED!");
			        Console.ReadKey();	// Wait for user to type a key (to allow them to read the message before the window is closed)
		            System.Environment.Exit(1);
		        }
	        }
        }
    }
}